from pathlib import Path

import pytest

from puddl.bash_history.parser import iter_bash_history


@pytest.fixture
def content():
    return """\
cat /tmp/something
#1585254366
cat <<EOF > /tmp/hello
world
EOF
#1585254441
tail ~/.bash_history
#1585254727
# this is a comment
echo and it's valid.
"""


@pytest.fixture
def bash_history_path(tmp_path, content) -> Path:
    path = tmp_path / 'example.bash_history'
    path.write_text(content)
    return path


def test_hist2dict(bash_history_path):
    with bash_history_path.open() as f:
        actual = list(iter_bash_history(f))
    multiline = """\
cat <<EOF > /tmp/hello
world
EOF\
"""
    expected = [
        (1585254366, multiline),
        (1585254441, 'tail ~/.bash_history'),
        (1585254727, "# this is a comment\necho and it's valid."),
    ]
    assert actual == expected



#!/usr/bin/env python
import os
from pathlib import Path

import structlog

from puddl.codimd import Remote, Driver
from puddl.codimd.index import Pickler

dir = Path('~/puddl/md').expanduser()
persistence = Pickler(dir / '_index.pickle')

log = structlog.get_logger()
remote = Remote(
    url=os.environ['CODIMD_URL'],
    email=os.environ['CODIMD_EMAIL'],
    password=os.environ['CODIMD_PASSWORD'],
)

driver = Driver(remote)

remote_docs = driver.history()
local_docs = persistence.read(fallback={})

new_ids = remote_docs.keys() - local_docs.keys()
outdated_ids = local_docs.keys() - remote_docs.keys()
if outdated_ids:
    log.info('found outdated. doing nothing. :>', count=len(outdated_ids))
maybe_update = remote_docs.keys() & local_docs.keys()


def iter_changed_docs():
    for x in new_ids:
        yield remote_docs[x]
    for key in maybe_update:
        rdoc = remote_docs[key]
        local = local_docs[key]
        if rdoc.time > local.time:
            log.info('new', doc=rdoc)
            yield rdoc


changed = iter_changed_docs()

for doc in changed:
    content = driver.download(doc.id)
    path = dir / f'{doc.id}.md'
    path.write_text(content)
    log.info('download', len=len(content), path=path, doc=doc)

persistence.write(remote_docs)



import pickle
from pathlib import Path

import structlog

from .models import Index

log = structlog.get_logger()


class Pickler:
    def __init__(self, path: Path):
        self.path = path

    def write(self, x: Index):
        with self.path.open('wb') as f:
            pickle.dump(x, f)

    def read(self, fallback=None) -> Index:
        try:
            with self.path.open('rb') as f:
                return pickle.load(f)
        except FileNotFoundError:
            log.info('no index')
            return fallback



import sqlalchemy
from sqlalchemy import text

from puddl.typing import URL


def list_databases(url: URL):
    engine = sqlalchemy.create_engine(url)
    with engine.connect() as conn:
        return [row[0] for row in conn.execute(text('SELECT datname FROM pg_catalog.pg_database')).fetchall()]


def list_schemas(url: URL):
    engine = sqlalchemy.create_engine(url)
    inspection_result = sqlalchemy.inspect(engine)
    return inspection_result.get_schema_names()



import pytest
from sqlalchemy import text

from puddl.pg import DB


def test_foo():
    import psycopg2.errorcodes
    from sqlalchemy.exc import ProgrammingError

    db = DB('psycoerror')

    with pytest.raises(ProgrammingError) as e:
        db.execute(text('CREATE TABLE foo (x INT)'))
        db.execute(text('CREATE TABLE foo (x INT)'))
        assert e.orig.pgcode == psycopg2.errorcodes.DUPLICATE_TABLE



from typing import Set, Dict

from pydantic import BaseModel, field_validator


class Remote(BaseModel):
    url: str  # TODO strip trailing slash
    email: str
    password: str


class Doc(BaseModel):
    id: str
    text: str
    time: int
    tags: Set

    # noinspection PyNestedDecorators
    @field_validator('tags')
    @classmethod
    def tags_as_set_for_hashing(cls, v) -> Set:
        return set(v)

    def __hash__(self):
        return self.model_dump_json().__hash__()


Index = Dict[int, Doc]



#!/usr/bin/env python

# normalize rotations
import base64
import re
from io import BytesIO

from PIL import Image
from sqlalchemy import text

from puddl.pg import DB

pdl = DB('exif')
conn = pdl.engine.connect()


def img_from_b64(s):
    data = re.sub('^data:image/.+;base64,', '', s)
    return Image.open(BytesIO(base64.b64decode(data)))


def b64_to_data_jpg(img):
    buf = BytesIO()
    img.save(buf, format='JPEG')
    return 'data:image/jpeg;base64,' + base64.b64encode(buf.getvalue()).decode()


if __name__ == '__main__':
    rows = conn.execute(text('SELECT id, thumb, rotation FROM markers WHERE rotation != 0'))
    for row in rows:
        img = img_from_b64(row.thumb).convert('RGB')
        # PIL.Image.rotate direction of rotation is counterclockwise
        # We have clockwise rotation in DB.
        pil_rotation = row.rotation * -1
        img = img.rotate(pil_rotation, expand=True)
        b64 = b64_to_data_jpg(img)
        conn.execute(
            text('UPDATE markers SET thumb=:thumb, rotation=0 WHERE id=:id'), parameters=dict(id=row.id, thumb=b64)
        )
    conn.commit()



import requests
import structlog

from puddl.codimd.models import Remote, Doc, Index

log = structlog.get_logger()


class CodiMDError(Exception):
    pass


def raise_on_alert_in_dom(response: requests.Response):
    """
    CodiMD seems not to know how to return non-200 HTTP status codes.
    So we look for "alert" in the DOM. *sigh*
    Could use beautiful soup or lxml for that, but meh.
    """
    o = response.text.find('alert')
    if o >= 0:
        raise CodiMDError(response.text[o : o + 200])  # noqa E203


class Driver:
    def __init__(self, remote: Remote):
        self.remote = remote
        self.s = requests.session()
        login = self.s.post(f'{remote.url}/login', data={'email': remote.email, 'password': remote.password})
        login.raise_for_status()

    def iter_history_raw(self):
        history = self.s.get(f'{self.remote.url}/history')
        history.raise_for_status()
        raise_on_alert_in_dom(history)
        records = history.json()['history']
        # [{'id': 'pycharm', 'text': 'pycharm', 'time': 1582037284079, 'tags': []}, ...]
        log.debug('history', records=records)
        for r in records:
            yield r

    def history(self) -> Index:
        docs = [Doc.model_validate(r) for r in self.iter_history_raw()]
        return {d.id: d for d in docs}

    def download(self, note_id: str) -> str:
        log.debug('download', note_id=note_id)
        r = self.s.get(f'{self.remote.url}/{note_id}/download')
        return r.text



from pathlib import Path

import click

import puddl.audio
from puddl.cli.base import root


@root.command(name='duration')
@click.argument('paths', type=Path, nargs=-1)
def duration(paths: [Path]):
    """
    Sum of audio duration of PATHS.
    """
    x = sum(puddl.audio.duration(path) for path in paths)
    print(x)
